// Author: Ji Prochzka
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.lcdui.List;
import java.io.*;
import java.util.*;

import nanoxml.*;

import org.kxml.*;
import org.kxml.parser.*;

public class XML extends MIDlet implements CommandListener {

    // members
    private Display display;
    //commands
    private Command okCommand = new Command("OK", Command.OK, 1);
    private Command backCommand = new Command("Zpt", Command.BACK, 1);
    private Command exitCommand = new Command("Konec", Command.EXIT, 1);

    //----------------------
    // Constructor
    //----------------------
    public XML() {
    }

    //----------------------
    // Public method
    //----------------------
    public void startApp() throws MIDletStateChangeException {
        if (display == null) display = Display.getDisplay(this);

        init();
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean b) throws MIDletStateChangeException {
    }

    // Command event handler
    public void commandAction(Command command, Displayable displayable) {
        if (command == exitCommand) {
            notifyDestroyed();
        } else if (command == okCommand) {

            // initialize raeder
            InputStreamReader reader = new InputStreamReader(getClass().getResourceAsStream("/test.xml"));

            // show XML in form
            switch (((List)displayable).getSelectedIndex()) {
                case 0:
                    display.setCurrent(getForm("kXML", parsekXML(reader)));
                    break;

                case 1:
                    display.setCurrent(getForm("Nano", parseNanoXML(reader)));
                    break;
            }
        } else if (command == backCommand) {
            // go back to XML parser menu
            display.setCurrent(getMenu());
        }
    }

    //----------------------
    // Private method
    //----------------------
    private void init() {
        display.setCurrent(getMenu());
    }

    /**
     * NanoXML parser.
     * @param reader
     * @return Vector of items
     */
    private Vector parseNanoXML(InputStreamReader reader) {
        Vector items = new Vector();

        XMLElement nanoParser = new XMLElement();

        try {
            nanoParser.parseFromReader(reader);
        } catch (IOException e) {
        } catch (XMLParseException e) {
        }

        Enumeration enum = nanoParser.enumerateChildren();

        while (enum.hasMoreElements()) {
            items.addElement(((XMLElement) enum.nextElement()).getContent());
        }

        return items;
    }

    /**
     * kXML parser.
     * @param reader
     * @return Vector of items
     */
    private Vector parsekXML(InputStreamReader reader) {

        Vector items = new Vector();
        boolean inItem = false;
        boolean perform = true;

        try {
            XmlParser parser = new XmlParser(reader);

            while (perform) {
                ParseEvent event = parser.read();
                switch (event.getType()) {
                    case Xml.START_TAG:
                        if (event.getName().equals("attr")) {
                            inItem = true;
                        }
                        break;
                    case Xml.END_TAG:
                        if (event.getName().equals("attr")) {
                            inItem = false;
                        }
                        break;
                    case Xml.TEXT:
                        if (inItem) {
                            items.addElement(event.getText());
                        }
                        break;

                    case Xml.END_DOCUMENT:
                        perform = false;
                        break;
                }
            }
        } catch (IOException e) {
        }

        return items;
    }

    // Retrieve XML parser list
    private List getMenu() {
        List menu = new List("Menu", List.IMPLICIT);

        menu.append("kXML", null);
        menu.append("Nano", null);

        menu.addCommand(okCommand);
        menu.addCommand(exitCommand);
        menu.setCommandListener(this);

        return menu;
    }

    // Retrieve form with items
    private Form getForm(String title, Vector items) {

        Form form = new Form(title);

        form.addCommand(backCommand);
        form.setCommandListener(this);

        Enumeration enum = items.elements();

        while (enum.hasMoreElements()) {
            form.append(new StringItem("", (String) enum.nextElement()));
        }

        return form;
    }
}
